; All these sources/objects are (c) Simon Brattel 1992

; Convert the title screen

		include ..\symbols.inc	; Define them
		include gsymbols.inc	;

CODE		SEGMENT public 		;

		ASSUME  CS:CODE		;

; Filenames

sSource		db "e:\dpaint2\pictures\title.lbm",0	; Title screen

sDest		db "s_salad1.ti0",0	; Dest

Start:		mov ax,StackSeg		; get Stack Segment Origin
		mov SS,ax		; and initialize the SS register
		mov SP,o TOS		; set SP to Top Of Stack

		; Free surplus RAM

		call Deallocate		; Release it

		; Now, read the source file

		call Read_Source	; Get it
		jc Exit			; Fail

		; Draw the file

		call Draw_Screen	; Draw into the screen buffer

		; Now, we have the file, convert it

		call Convert		; Convert it

		; Create the title-file

		call Open_Title		; Create the file

		; Close the file
	
		call WF_Close		; End it

	; The exit message

Exit:		mov ah,terminate	;
		int dos			; Done

; Convert the file

Convert:	mov ax,DATA2		; The screen
		mov ds,ax		;

		mov si,o Screen		; Point at the display buffer

		; Now, force a break after the last pixel

		mov di,o Screen_End	; Point at the end

		mov al,-1[di]		; Last pixel
		inc al			; Different value
		mov [di],al		;

		; Convert it

		mov ax,DATA1		; The file
		mov es,ax		;

		mov di,o Title_Data	; The title screen

		mov bx,-1		; Illegal ptr

		; Compress the screen

Conv_Lp:	cmp si,o Screen_End	; End ?
		je Conv_End		; Yes, done

		mov al,[si]		; Get the next colour
		inc si			;

		mov cx,1		; One pixel

Conv_Lp1:	cmp al,[si]		; Same colour ?
		jne Conv_Next		; No, skip

		; Yes, try again

		inc si			; Skip it
		inc cx			; One more

		jmp s Conv_Lp1		; Loop

	; We have al = colour, cx = length

Conv_Next:	cmp bx,-1		; First ptr ?
		je CN_First		; Yes, plant it

		; Now, were we in a run ?

		mov ah,es:[bx]		; Is this a run of different pixels ?
		cmp ah,124		;
		jae CN_New		; Not a run, or no room, plant as new

		; We have a run, should we add to it ?

		cmp cx,2		; Less than 3 ?
		jae CN_New		; No, add it

		; Yes, assume 1

CN_Extend:	mov es:[di],al		; New colour
		inc di			;

		inc b es:[bx]		; Step

		loop CN_Extend		; Perhaps do it again

		; Done, loop

		jmp s Conv_Lp		; Loop

	; We must plant a new block

CN_New:		cmp cx,1		; Is this a single pixel ?
		jne CN_First		; No, plant it as a block

		; Yes, plant a run

		mov bx,di		; Save the ptr

		mov es:[di],cl		; Count := 1
		inc di			;

		mov es:[di],al		; Colour
		inc di			;

		; Ignore

		jmp Conv_Lp		; Loop

	; Plant the first block

CN_First:	call Plant_Multi	; Plant a multi-byte

		jmp s Conv_Lp		; Loop

	; Plant a repeated colour

Plant_Multi:	mov bx,di		; Point at this multi-count

		cmp cx,128		; In range ?
		ja PM_Next		; No, skip

		neg cl			; Plant it as a negative
		mov es:[di],cl		; Save the count
		inc di			;

		mov es:[di],al		; Save the colour
		inc di			;

		ret			; Done

PM_Next:	mov b es:[di],-128	; Count
		inc di			;

		mov es:[di],al		; Colour
		inc di			;

		sub cx,128		; This many done

		jmp s Plant_Multi	; Next block

	; Done, save the length

Conv_End:	mov b es:[di],0		; Terminate it
		inc di			;

		sub di,o Title_Data	; calc Length
		mov cs:Title_Length,di	; End value

		ret			; Done

; Now, draw the screen

Draw_Screen:	mov ax,DATA2		; The data-area
		mov ds,ax		;

		; Now draw the picture

		mov si,03E8h+o File	; HACK ! this is the start of the BODY

		mov di,o Screen		; Point at the buffer

		mov cx,200		; Read this many lines

RB_Lp1:		push cx			; Save

		call Read_Line		; Read a pixel line

		pop cx			; Restore
		loop RB_Lp1		; Loop

		ret			; Done

; Read a pixel-line

Read_Line:	lea bp,320[di]		; Point at the end address

		mov dl,01h		; This bit
		mov dh,00h		; Mask

RL_Lp:		push di			; Read a block
		call Read_Line_Bit	; Read a bit
		pop di			;

		mov dh,0FFh		; New mask

		add dl,dl		; Shift the bit
		jnc RL_Lp		; Loop

		; Skip this line

		mov di,bp		; Skip to the next line

		ret			; Done

	; Read a line of bits

RLB_Skip:	inc si			; Skip

Read_Line_Bit:	mov al,es:[si]		; Get the next byte of the file
		inc si			; Skip

		cmp al,-127		; Ignore ?
		je RLB_Skip		; Yes, loop

		test al,080h		; -ve ?
		jnz RLB_Neg		; Yes, skip

		; Copy the next (n+1) bytes

		inc al			; n+1

RLB_C_Lp:	mov ah,es:[si]		; Copy a byte
		inc si			;

		call RLB_Write		; Write 8 bits

		dec al			; Loop
		jnz RLB_C_Lp		;

		; End of line ?

RLB_EOL:	cmp di,bp		; EOL ?
		jnz Read_Line_Bit	; No, loop

		ret			; Done

	; Copy the next byte (n+1) times

RLB_Neg:	neg al			; Negate it
		inc al			; n+1

		mov ah,es:[si]		; Copy a byte
		inc si			;

RLB_D_Lp:	call RLB_Write		; Write 8 bits

		dec al			; Loop
		jnz RLB_D_Lp		;

		jmp s RLB_EOL		; Skip

	; Write 8 bits

RLB_Write:	push ax			; Save

		mov al,8		; This many bits

RLBW_Lp:	mov bl,ds:[di]		; Mask it
		and bl,dh		;

		add ah,ah		; Test the top bit
		jnc RLBW_Z		; No, skip

		or bl,dl		; Merge it		

RLBW_Z:		mov ds:[di],bl		; Write it
		inc di			; Skip

		dec al			; Loop
		jnz RLBW_Lp		;

		pop ax			; Restore
		ret			; Done

; Read the source file

Read_Source:	mov dx,o sSource	; Get the ^string

RS_Read:	push cs			; ds:dx
		pop ds			;

		mov bx,o File		; Point at the map
		mov cx,8		; Length

		mov ax,DATA1		; The segment
		mov es,ax		;

		push dx			; Save
		call Read_File		; Read it
		pop dx			; Restore

		jc RF_Exit		; Can't, exit

		; Now, get the length

		mov bx,o File + 6	; Length
		mov cx,es:[bx]		; Get the length
		xchg cl,ch		; Order them

		add cx,8		; Length (total)

		; Now read the file

		push cs			; ds:dx
		pop ds			;

		mov bx,o File		; Point at the map

		mov ax,DATA1		; The segment
		mov es,ax		;

		call Read_File		; Read it
		jc RF_Exit		; Can't, exit

		clc			; Ok

RF_Exit:	ret			; Done

; Write the destn file

Open_Title:	mov dx,o sDest		; Get the ^string

		push cs			; Use ds
		pop ds			;

		mov bx,o Title_Data	; Point at the title

		mov cx,cs:Title_Length	; Length

		mov ax,DATA1		; The data segment
		mov es,ax		;

		call Write_File		; Write it

		ret			; Done

; Possible read filename dx:ds, length cx, data from es:bx

Read_File:	mov cs:WF_Len,cx	; Save the length

		mov cs:WF_Off,bx	; Start of data

		mov ax,es		; Save the seg
		mov cs:WF_Seg,ax	;

		mov cx,00h		; File attribute

		mov al,0		; Read only

		mov ah,OpenFile		; Call dos
		int dos			;

		jc PRF_Exit		; Can't open file

		mov cs:ReadHandle,ax	; Save the handle

		; Rewind to start

		mov cx,0		; Rewind file
		mov dx,0		;

		mov bx,ax		; Handle

		mov al,0		;

		mov ah,MovePointer	;
		int dos			;

		; Read the file

		mov ax,cs:WF_Seg	; Segment
		mov ds,ax		;

		mov bx,cs:WF_Off	; Offset

		mov cx,cs:WF_Len	; Length

		call Read_Block		; Write it

		; Close the file

		push ax			; Save error code

		mov bx,cs:ReadHandle	; Close it
		mov ah,CloseFile	;
		int dos			;

		pop ax			;

		; Done

PRF_Exit:	ret			; Done

; Write filename dx:ds, length cx, data from es:bx

Write_File:	mov cs:WF_Len,cx	; Save the length

		mov cs:WF_Off,bx	; Start of data

		mov ax,es		; Save the seg
		mov cs:WF_Seg,ax	;

		mov cx,00h		; File attribute

		mov ah,CreateFile	; Call dos
		int dos			;

		jc WF_Fail		; Can't create file

		mov cs:TempHandle,ax	; Save the handle

		; Write the keys

		mov ax,cs:WF_Seg	; Segment
		mov ds,ax		;

		mov bx,cs:WF_Off	; Offset

		mov cx,cs:WF_Len	; Length

		call Save_Block		; Write it

		; Done

WF_Fail:	ret			; Done

; Re-wind the file

WF_Rewind:	mov cx,0		; Rewind file
		mov dx,0		;

		mov bx,cs:TempHandle	; Handle

		mov al,0		;

		mov ah,MovePointer	;
		int dos			;

		ret			; Done

; Close the file

WF_Close:	push ax			; Save error code

		mov bx,cs:TempHandle	; Close it
		mov ah,CloseFile	;
		int dos			;

		pop ax			;

		ret			; Done

; Write block ds:bx for length cx

Save_Block:	push ax			; Save reg
		push dx			;

		mov dx,bx		; Get ptr

		mov bx,cs:TempHandle	; Get file handle

		mov ah,WriteToFile	; Write it
		int dos			;
		jc SB_Fail		; Fail

		cmp ax,cx		; Save full length ?
		jz SB_OK		; Yes, skip

		stc			; Assert fail

SB_Fail:	pop dx			; Restore
		pop ax			; dummy

		mov al,64		; "Disc full"

		ret			;

SB_OK:		pop dx			; Restore
		pop ax			;

		ret			;

; Read block ds:bx for length cx

Read_Block:	push ax			; Save reg
		push dx			;

		mov dx,bx		; Get ptr

		mov bx,cs:ReadHandle	; Get file handle

		mov ah,ReadFromFile	; Read it
		int dos			;
		jc SB_Fail		; Fail

		cmp ax,cx		; Save full length ?
		jz SB_OK		; Yes, skip

		stc			; Assert fail

		jmp SB_Fail		; Fail

; Vars

WF_Len		dw ?			; Length
WF_Seg		dw ?			; Segment
WF_Off		dw ?			; Offset

TempHandle	dw ?			; Handle
ReadHandle	dw ?			; Handle for reading

; Release the unused RAM

Deallocate:	mov bx,o TOS		; Point at top of ss

		mov cl,4		; Convert to paragraphs
		shr bx,cl		;
		inc bx			; add one for luck

		mov cx,ss		; Get stack segment
		add bx,cx		; Add stack segment
		
		mov cx,es		; Get extra segment
		sub bx,cx		; Subtract it

		; bx contains no. of paragraphs from es to the top of ss

		mov ah,Set_Block	; Deallocate surplus RAM
		int dos			;

		mov al,9		; "Internal memory"
		ret			;

; Vars

Title_Length	dw ?			; Length of the file

		;

CODE		ends

; Data

DATA1		SEGMENT byte public	; A segment for the file

Title_Data	equ this byte
File		db 65535 dup (?)	; Load it into here

DATA1		ends			; End it

DATA2		SEGMENT byte public	; A segment for the graphics

Screen		db 320*200 dup (?)	; Draw it into here
Screen_End	db ?			;

DATA2		ends			; End it

StackSeg	segment stack

		dw 400 dup (?)
TOS		equ this word

StackSeg	ends

		end Start

